<?php

namespace Drupal\feature_toggle\Drush\Commands;

use Drupal\feature_toggle\FeatureInterface;
use Drupal\feature_toggle\FeatureManagerInterface;
use Drupal\feature_toggle\FeatureStatusInterface;
use Drush\Attributes as CLI;
use Drush\Commands\DrushCommands;

// cspell:ignore ftset

/**
 * Drush commands for feature_toggle.
 */
final class FeatureToggleCommands extends DrushCommands {

  /**
   * Constructs a new FeatureToggleCommands object.
   *
   * @param \Drupal\feature_toggle\FeatureManagerInterface $featureManager
   *   The feature manager.
   * @param \Drupal\feature_toggle\FeatureStatusInterface $featureStatus
   *   The feature status.
   */
  public function __construct(
    protected FeatureManagerInterface $featureManager,
    protected FeatureStatusInterface $featureStatus,
  ) {
    parent::__construct();
  }

  /**
   * Sets a specific value for a given feature.
   *
   * @param string $name
   *   The feature name.
   * @param string|int $value
   *   The feature value. It has to be 0 or 1.
   */
  #[CLI\Command(name: 'feature_toggle:set', aliases: ['ftset'])]
  #[CLI\Argument(name: 'name', description: 'The feature machine name.')]
  #[CLI\Argument(name: 'value', description: 'The toggle value (0 = off, 1 = on).')]
  public function toggle(string $name, string|int $value): void {
    // Validate feature using featureManager.
    $feature = $this->featureManager->getFeature($name);
    if (!$feature) {
      $features = $this->featureManager->getFeatures();
      $names = array_map(fn(FeatureInterface $feature) => $feature->name(), $features);
      throw new \InvalidArgumentException("Feature '{$name}' does not exist. Available features are: " . implode(', ', array_values($names)));
    }

    // Validate flag value.
    if (!is_numeric($value) || !in_array((int) $value, [0, 1], TRUE)) {
      throw new \InvalidArgumentException('Flag must be 0 or 1.');
    }

    $this->featureStatus->setStatus($feature, (bool) $value);
    $this->output()->writeln("Feature '{$name}' toggled to {$value}.");
  }

}
