<?php

namespace Drupal\feature_toggle\Plugin\Condition;

use Drupal\Core\Condition\ConditionPluginBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\feature_toggle\FeatureInterface;
use Drupal\feature_toggle\FeatureManagerInterface;
use Drupal\feature_toggle\FeatureStatusInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'Feature Toggle' condition.
 *
 * @Condition(
 *   id = "feature_toggle",
 *   label = @Translation("Feature Toggle"),
 * )
 */
class FeatureToggle extends ConditionPluginBase implements ContainerFactoryPluginInterface {

  /**
   * The feature list array.
   *
   * @var array
   */
  protected $features;

  /**
   * The list of enabled features.
   *
   * @var array
   */
  protected $enabledFeatures;

  /**
   * Creates a new FeatureToggle instance.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\feature_toggle\FeatureManagerInterface $featureManager
   *   The feature manager.
   * @param \Drupal\feature_toggle\FeatureStatusInterface $featureStatus
   *   The feature status.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    protected FeatureManagerInterface $featureManager,
    protected FeatureStatusInterface $featureStatus,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->setConfiguration($configuration);
    $features = $this->featureManager->getFeatures();

    $features_temp = $enabled_temp = [];
    array_walk($features, function (FeatureInterface $value) use (&$features_temp, &$enabled_temp) {
      $features_temp[$value->name()] = $value->label();
      if ($this->featureStatus->getStatus($value->name())) {
        $enabled_temp[] = $value->name();
      }
    });

    $this->features = $features_temp;
    $this->enabledFeatures = $enabled_temp;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('feature_toggle.feature_manager'),
      $container->get('feature_toggle.feature_status')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['features'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('When the following features are enabled'),
      '#default_value' => $this->configuration['features'],
      '#options' => array_map('\Drupal\Component\Utility\Html::escape', $this->features),
      '#description' => $this->t('If you select no features, the condition will evaluate to TRUE.'),
    ];
    return parent::buildConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'features' => [],
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['features'] = array_filter($form_state->getValue('features'));
    parent::submitConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function summary() {
    // Use the feature labels. They will be sanitized below.
    $features = array_intersect_key($this->features, $this->configuration['features']);
    if (count($features) > 1) {
      $features = implode(', ', $features);
    }
    else {
      $features = reset($features);
    }
    if (!empty($this->configuration['negate'])) {
      return $this->t('One of these features @features are not enabled', ['@features' => $features]);
    }
    else {
      return $this->t('One of these features @features are enabled', ['@features' => $features]);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function evaluate() {
    if (empty($this->configuration['features']) && !$this->isNegated()) {
      return TRUE;
    }
    return (bool) array_intersect($this->configuration['features'], $this->enabledFeatures);
  }

}
