<?php

namespace Drupal\feature_toggle\Plugin\views\access;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\feature_toggle\FeatureManagerInterface;
use Drupal\feature_toggle\FeatureStatusInterface;
use Symfony\Component\Routing\Route;

/**
 * Common functionality for Feature based views access checkers.
 */
trait ViewsFeatureAccessTrait {

  /**
   * The feature manager.
   *
   * @var \Drupal\feature_toggle\FeatureManagerInterface
   */
  protected FeatureManagerInterface $featureManager;

  /**
   * The feature status.
   *
   * @var \Drupal\feature_toggle\FeatureStatusInterface
   */
  protected FeatureStatusInterface $featureStatus;

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();

    $options['feature_name'] = ['default' => ''];
    $options['status'] = ['default' => TRUE];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    $options = [];
    $features = $this->featureManager()->getFeatures();
    foreach ($features as $feature) {
      $options[$feature->name()] = $feature->label();
    }

    $form['feature_name'] = [
      '#type' => 'select',
      '#options' => $options,
      '#title' => $this->t('Feature'),
      '#default_value' => $this->options['feature_name'],
      '#description' => $this->t('Only Tier checkers that matches will be able to access this display.'),
      '#empty_option' => $this->t('- None -'),
    ];

    $form['status'] = [
      '#type' => 'radios',
      '#title' => $this->t('Feature status'),
      '#options' => [
        1 => $this->t('Enabled'),
        0 => $this->t('Disabled'),
      ],
      '#default_value' => $this->options['status'],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function access(AccountInterface $account) {
    // @phpstan-ignore-next-line
    $access = !is_callable([parent::class, 'access']) || parent::access($account);

    return $access && (empty($this->options['feature_name']) ||
      ($this->featureStatus()->getStatus($this->options['feature_name']) == $this->options['status']));
  }

  /**
   * {@inheritdoc}
   */
  public function alterRouteDefinition(Route $route) {
    if (is_callable([parent::class, 'alterRouteDefinition'])) {
      // @phpstan-ignore-next-line
      parent::alterRouteDefinition($route);
    }

    if (!empty($this->options['feature_name'])) {
      $slug = $this->options['status'] ? 1 : 0;
      $route->setRequirement('_feature_toggle', $this->options['feature_name'] . '.' . $slug);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    return empty($this->options['feature_name']) ?
      ['feature_toggle_list'] :
      ['feature_toggle:' . $this->options['feature_name']];
  }

  /**
   * Gets the feature manager.
   *
   * @return \Drupal\feature_toggle\FeatureManagerInterface
   *   The feature manager.
   */
  protected function featureManager(): FeatureManagerInterface {
    if (!isset($this->featureManager)) {
      $this->featureManager = \Drupal::service('feature_toggle.feature_manager');
    }

    return $this->featureManager;
  }

  /**
   * Gets the feature status.
   *
   * @return \Drupal\feature_toggle\FeatureStatusInterface
   *   The feature status.
   */
  protected function featureStatus(): FeatureStatusInterface {
    if (!isset($this->featureStatus)) {
      $this->featureStatus = \Drupal::service('feature_toggle.feature_status');
    }

    return $this->featureStatus;
  }

}
